@@ -153,13 +153,14 @@ print.summary.bayesPop.prediction <- function(x, digits = 5, ...) {
 	if(!is.null(x$country.name)) {
 		cat('\n\nCountry:', x$country.name, '\n')
 		cat('\nProjected Population')
-		if (x$sex != 'all') cat(' for', x$sex)
+		if (x$sex != 'both') cat(' for', x$sex)
 		cat(':\n')
 		print(x$projections, digits=digits, ...)
 	}
 }
 
 get.pop.observed.with.age <- function(pop.pred, country, sex=c('both', 'male', 'female'), age='all', data=NULL) {
+	# Results are not sorted in the same order as values in "country". The caller should take care of it. 
 	sex <- match.arg(sex)
 	if(is.null(data)) data <- pop.pred$inputs$pop.matrix
 	if(sex == 'both') {
@@ -191,19 +192,22 @@ get.pop.observed <- function(pop.pred, country, sex=c('both', 'male', 'female'),
 }
 
 get.pop.observed.multiple.countries <- function(pop.pred, countries, sex=c('both', 'male', 'female'), age='all', sum.over.ages=TRUE) {
+	# the function assumes that population data are sorted be age
 	data.age <- get.pop.observed.with.age(pop.pred, countries, sex, age)
 	data <- data.age$data
 	age.idx <- data.age$age.idx
 	ncountries <- length(countries)
 	max.age <- data.age$max.age
-	cindex <- seq(1, length=ncountries, by=max.age)
+	cindex <- lapply(countries, function(x) grep(paste0("^", x, "_"), rownames(data), value=FALSE)[1:max.age])
+	names(cindex) <- countries
 	if(age[1]=='psr')  # potential support ratio
-		return(list(data=t(sapply(cindex, function(start.idx) 
-				colSums(data[start.idx:(start.idx+max.age),][get.psr.nominator.index(),])/colSums(data[start.idx:(start.idx+max.age),][get.psr.denominator.startindex():max.age,]))), age.idx=age.idx))
-	sum.over.countries <- function(start.idx) return(colSums(data[start.idx:(start.idx+max.age),][age.idx,]))
+		return(list(data=t(sapply(cindex, function(country.idx) 
+				colSums(data[country.idx,][get.psr.nominator.index(),])/colSums(data[country.idx,][get.psr.denominator.startindex():max.age,]))), age.idx=age.idx))
+	sum.over.countries <- function(country.idx) return(colSums(data[country.idx,][age.idx,]))
 	if(sum.over.ages) return(list(data=t(sapply(cindex, sum.over.countries)), age.idx=age.idx))
 	res <- array(NA, c(ncountries, length(age.idx), ncol(data)), dimnames=list(countries, age.idx, colnames(data)))
-	for(i in 1:length(age.idx)) res[,i,] <- as.matrix(data[seq(age.idx[i], length=ncountries, by=max.age),])
+	countries.char <- names(cindex)
+	for(i in 1:ncountries) res[i,,] <- as.matrix(data[cindex[[countries.char[i]]],])
 	return(list(data=res, age.idx=age.idx))
 }
 
@@ -214,18 +218,30 @@ get.psr.denominator.startindex <- function() return(14)
 .get.pop.quantiles <- function(pop.pred, what='', adjust=FALSE) {
 	quant <- pop.pred[[paste0('quantiles', what)]]
 	if(!adjust) return(quant)
-	return(adjust.quantiles(quant, what, env=pop.pred$adjust.env))
+	return(adjust.quantiles(quant, what, wpp.year=pop.pred$wpp.year, env=pop.pred$adjust.env))
+}
+
+.load.traj.file <- function(dir, country, e) {
+	traj.file <- file.path(dir, paste('totpop_country', country, '.rda', sep=''))
+	if (!file.exists(traj.file)) return(0)
+	load(traj.file, envir=e)
+	return(1)
+}
+
+pop.trajectories <- function(pop.pred, country, sex=c('both', 'male', 'female'), age='all', ...) {
+	country.object <- get.country.object(country, country.table=pop.pred$countries)
+	res <- get.pop.trajectories(pop.pred, country.object$code, sex=sex, age=age, ...)
+	return(res$trajectories[,res$index])
 }
 
 get.pop.trajectories <- function(pop.pred, country, sex=c('both', 'male', 'female'), age='all',
  									nr.traj=NULL, typical.trajectory=FALSE, adjust=FALSE) {
-	traj.file <- file.path(pop.output.directory(pop.pred), paste('totpop_country', country, '.rda', sep=''))
+	
 	quant <- hch <- age.idx <- traj <- traj.idx <-  NULL
 	load.traj <- is.null(nr.traj) || nr.traj > 0 || typical.trajectory
-	if (!file.exists(traj.file)) 
-		return(list(trajectories=traj, index=traj.idx, quantiles=quant, age.idx=age.idx, half.child=hch))
 	e <- new.env()
-	load(traj.file, envir=e)
+	if (!.load.traj.file(pop.output.directory(pop.pred), country, e))
+		return(list(trajectories=traj, index=traj.idx, quantiles=quant, age.idx=age.idx, half.child=hch))
 	if(adjust) {
 		if(is.null(pop.pred$adjust.env)) pop.pred$adjust.env <- new.env()
 		adjust.trajectories(country, e, pop.pred, pop.pred$adjust.env)
@@ -234,7 +250,7 @@ get.pop.trajectories <- function(pop.pred, country, sex=c('both', 'male', 'femal
 	max.age <- dim(e$totpf)[1] # should be 27
 	age.idx <- if(age[1]=='all' || age[1]=='psr') 1:max.age else age
 	if(max(age.idx) > 27 || min(age.idx) < 1) stop('Age index must be between 1 (age 0-4) and 27 (age 130+).')
-	if(sex == 'both' && age[1]=='all') {
+	if(sex == 'both' && all((1:max.age) %in% age.idx)) { # for both sexes and all ages
 		if(load.traj) traj <- e$totp
 		quant <- .get.pop.quantiles(pop.pred, adjust=adjust)
 		hch <- e$totp.hch
@@ -284,16 +300,15 @@ get.pop.trajectories <- function(pop.pred, country, sex=c('both', 'male', 'femal
 	return(list(trajectories=traj, index=traj.idx, quantiles=quant, age.idx=age.idx, half.child=hch))
 }
 
+
 get.pop.trajectories.multiple.age <- function(pop.pred, country, sex=c('both', 'male', 'female'), 
 												age='all', nr.traj=NULL, proportion=FALSE, typical.trajectory=FALSE,
 												adjust=FALSE) {
 	# Like get.pop.trajectories() but it doesn't sum up over ages.
 	# Called when creating pop pyramid and pop.byage.*. Doesn't handle potential support ratio.
-	traj.file <- file.path(pop.output.directory(pop.pred), paste('totpop_country', country, '.rda', sep=''))
 	age.idx <- traj.idx <- traj <- quant <- hch <- NULL
-	if (file.exists(traj.file)) {
-		e <- new.env()
-		load(traj.file, envir=e)
+	e <- new.env()
+	if (.load.traj.file(pop.output.directory(pop.pred), country, e)) {
 		sex <- match.arg(sex)
 		max.age <- dim(e$totpm)[1] # should be 27
 		age.idx <- if(age[1]=='all') 1:max.age else age
@@ -334,7 +349,7 @@ is.saved.pi <- function(pop.pred, pi, warning=TRUE) {
 	is.valid.pi <- rep(NA, length(pi))
 	quantile.values <- as.numeric(dimnames(pop.pred$quantiles)[[2]])
 	for (i in 1:length(pi)) {
-		al <- 1-(1-pi[i]/100)/2		
+		al <- (1+pi[i]/100)/2		
 		is.valid.pi[i] <- any(round(quantile.values,6)==round(al,6))
 		if(!is.valid.pi[i] && warning)
 			warning(pi[i], '% interval not available.')
@@ -345,7 +360,7 @@ is.saved.pi <- function(pop.pred, pi, warning=TRUE) {
 get.pop.traj.quantiles.byage <- function(quantile.array, pop.pred, country.index, country.code, year.index,
 									trajectories=NULL, pi=80, q=NULL, ...) {
 	# quantile.array should be 4d-array (country x age x quantiles x time)
-	al <- if(!is.null(q)) q else c((1-pi/100)/2, 1-(1-pi/100)/2)
+	al <- if(!is.null(q)) q else c((1-pi/100)/2, (1+pi/100)/2)
 	found <- FALSE
 	if(!is.null(quantile.array)) {
 		quantile.values <- as.numeric(dimnames(quantile.array)[[3]])
@@ -376,7 +391,7 @@ get.pop.traj.quantiles <- function(quantile.array, pop.pred, country.index=NULL,
 									trajectories=NULL, pi=80, q=NULL, reload=TRUE, ...) {
 	# quantile.array should be 3d-array (country x quantiles x time). 
 	# If country.index is NULL, the country dimension can be omitted 
-	al <- if(!is.null(q)) q else c((1-pi/100)/2, 1-(1-pi/100)/2)
+	al <- if(!is.null(q)) q else c((1-pi/100)/2, (1+pi/100)/2)
 	found <- FALSE
 	if(!is.null(quantile.array)) {
 		if(is.null(country.index) && length(dim(quantile.array))<3) {
@@ -487,7 +502,7 @@ get.survival <- function(mxm, sex, age05=c(FALSE, FALSE, TRUE)) {
 		LLm <- LifeTableMxCol(mxm[,, itraj, drop=FALSE], colname='Lx', sex=sex, age05=age05)
 		if(itraj == 1) {
 			if(is.null(dim(LLm))) LLm <- abind(LLm, along=2)
-			sx <- array(0, dim=c(dim(LLm)[1], dim(LLm)[2], dim(mxm)[3]))
+			sx <- array(0, dim=c(dim(LLm)[1], dim(LLm)[2], dim(mxm)[3]), dimnames=vector("list", 3))
 			sr <- rep(0, dim(LLm)[1])
 		}
 		sx[,, itraj] <- apply(LLm, 2, 
@@ -610,10 +625,15 @@ get.popVE.trajectories.and.quantiles <- function(pop.pred, country,
 			if(has.hch) hch <- alltraj[[paste(sex,'hch', sep='.')]][age.idx,,,drop=FALSE]
 		}
 	}
-	if(is.observed && dim(traj)[[2]] < nperiods)
-		traj <- abind(array(NA, dim=c(dim(traj)[[1]], nperiods-dim(traj)[[2]], dim(traj)[[3]]), 
+	if(is.observed) {
+		if(length(dim(traj)) < 3) # age dimension is missing
+			traj <- abind(traj, NULL, along=0)
+		 if(dim(traj)[[2]] < nperiods) {		
+			traj <- abind(array(NA, dim=c(dim(traj)[[1]], nperiods-dim(traj)[[2]], dim(traj)[[3]]), 
 						dimnames=list(NULL, colnames(pop.pred$inputs$pop.matrix$male)[1:(nperiods-dim(traj)[[2]])], NULL)),
 						traj, along=2)
+		}
+	}
 	quant <- NULL
 	if(!is.observed) {
 		if(sum.over.ages) { # quantiles are 2-d arrays
@@ -1133,6 +1153,11 @@ drop.age <- function(data) {
 age.index01 <- function(end) return (c(-1,0,2:end))
 age.index05 <- function(end) return (1:end)
 	
+mac.expression <- function(country) {
+	factors <- seq(17.5, by=5, length=7)
+	return(paste0("(", paste0(factors, "*R", country, "[", 4:10, "]", collapse=" + "), ")/100"))
+}
+	
 .solve.expression.for.country <- function(icountry, pop.pred, expression, adjust=FALSE) {
 	country <- pop.pred$countries$code[icountry]
 	expr <- gsub('XXX', as.character(country), expression, fixed=TRUE)
@@ -1241,4 +1266,54 @@ as.environment.bayesPop.prediction <- function(pop.pred){
 
 UNcountries <- function()
 	return(UNlocations$country_code[UNlocations$location_type==4])
-	
\ No newline at end of file
+	
+cohorts <- function(pop.pred, country=NULL, expression=NULL, pi=c(80, 95)) {
+	.get.quantiles.from.cohort.data <- function(trajs) {
+		res <- apply(trajs, 1, median)
+		res <- rbind(res, apply(trajs, 1, quantile, quants))
+		rownames(res) <- c("median", quants.char)
+		res
+	}
+	if(is.null(country) && is.null(expression))
+		stop("Either country or expression must be given.")
+	
+	if(!is.null(country)) {
+		country.object <- get.country.object(country, country.table=pop.pred$countries)
+		expression <- paste0("P", country.object$code, "{}")
+	}
+	alldata <- get.pop.trajectories.from.expression.multiple.age(expression, pop.pred)$trajectories # should be 3d array
+	quants <- sort(c((1-pi/100)/2, (1+pi/100)/2))
+	quants.char <- as.character(quants)
+	result <- list()
+	age.index <- as.integer(dimnames(alldata)[[1]])
+	nage <- dim(alldata)[1]
+	years <- dimnames(alldata)[[2]]
+	last.observed.cohort <- pop.pred$proj.years.pop[1]-age.index[1]*5
+	from.cohorts <- seq(last.observed.cohort, length=nage-1, by=-5)
+	observed.cohorts <- paste(from.cohorts, '-', from.cohorts+5, sep="")
+	for(cohort in length(observed.cohorts):1) {
+		cohort.traj <- apply(alldata[cohort:nage,,,drop=FALSE], 3, 'diag')
+		result[[observed.cohorts[cohort]]] <- .get.quantiles.from.cohort.data(cohort.traj)
+		colnames(result[[observed.cohorts[cohort]]]) <- years[1:ncol(result[[observed.cohorts[cohort]]])]
+	}
+	nyears <- length(pop.pred$proj.years.pop)
+	from.cohorts <- seq(last.observed.cohort + 5, length=nyears-2, by=5)
+	projected.cohorts <- paste(from.cohorts, '-', from.cohorts+5, sep="")	
+	for(cohort in 1:length(projected.cohorts)) {
+		cohort.traj <- apply(alldata[,(cohort+1):nyears,,drop=FALSE], 3, 'diag')
+		result[[projected.cohorts[cohort]]] <- .get.quantiles.from.cohort.data(cohort.traj)
+		colnames(result[[projected.cohorts[cohort]]]) <- years[(cohort+1):(cohort+ncol(result[[projected.cohorts[cohort]]]))]
+	}
+	#stop('')
+	result[["last.observed"]] <- last.observed.cohort
+	return(result)
+}
+
+get.trajectory.indices <- function(pop.pred, country, what=c("TFR", "e0M", "e0F", "migM", "migF")) {
+	country.object <- get.country.object(country, country.table=pop.pred$countries)
+	e <- new.env()
+	if (!.load.traj.file(pop.output.directory(pop.pred), country.object$code, e))
+		return (NULL)
+	par <- list(TFR="TFRpred", e0M="e0Mpred", e0F="e0Fpred", migM="migMpred", migF="migFpred")
+	return(e$trajectory.indices[[par[[what[1]]]]])
+}
\ No newline at end of file
