@@ -37,10 +37,10 @@ void doLifeTable(int sex, int nage, double *mx,
 		}
 	}
 	qx[0] = mx[0] / (1 + (1 - ax[0]) * mx[0]);                    /* 1q0 */	
-	qx[1] = (4 - ax[1]) * mx[1];								  /* 4q1 */	
+	qx[1] = 4 * mx[1] / (1 + (4 - ax[1]) * mx[1]);				  /* 4q1 */	
 	lx[0] = 1;                                                    /* l0 */
 	lx[1] = lx[0] * (1 - qx[0]);                                  /* l1 */
-	lx[2] = lx[1] * (1 - 4 * mx[1] / (1 + qx[1]));                 /* l5 = l1 * (1-4q1) */
+	lx[2] = lx[1] * (1 - qx[1]);                                  /* l5 = l1 * (1-4q1) */
 	Lx[0] =  lx[1] + ax[0] * (lx[0] - lx[1]);                     /* 1L0 */
 	Lx[1] =  4 * lx[2] + ax[1] * (lx[1] - lx[2]);                 /* 4L1 */
 		
